#!/usr/bin/env python3

# DRController, use the UsendMii Client on Linux !
import socket
import ast
import enum
import libevdev
import time

class VPADButtons(enum.IntEnum): # VPADButtons bitmasks
   VPAD_BUTTON_A = 0x8000
   VPAD_BUTTON_B = 0x4000
   VPAD_BUTTON_X = 0x2000
   VPAD_BUTTON_Y = 0x1000
   VPAD_BUTTON_LEFT = 0x0800
   VPAD_BUTTON_RIGHT = 0x0400
   VPAD_BUTTON_UP = 0x0200
   VPAD_BUTTON_DOWN = 0x0100
   VPAD_BUTTON_ZL = 0x0080
   VPAD_BUTTON_ZR = 0x0040
   VPAD_BUTTON_L = 0x0020
   VPAD_BUTTON_R = 0x0010
   VPAD_BUTTON_PLUS = 0x0008
   VPAD_BUTTON_MINUS = 0x0004
   VPAD_BUTTON_HOME = 0x0002
   VPAD_BUTTON_SYNC = 0x0001
   VPAD_BUTTON_STICK_R = 0x00020000
   VPAD_BUTTON_STICK_L = 0x00040000
   VPAD_BUTTON_TV = 0x00010000
   VPAD_STICK_R_EMULATION_LEFT = 0x04000000
   VPAD_STICK_R_EMULATION_RIGHT = 0x02000000
   VPAD_STICK_R_EMULATION_UP = 0x01000000
   VPAD_STICK_R_EMULATION_DOWN = 0x00800000
   VPAD_STICK_L_EMULATION_LEFT = 0x40000000
   VPAD_STICK_L_EMULATION_RIGHT = 0x20000000
   VPAD_STICK_L_EMULATION_UP = 0x10000000
   VPAD_STICK_L_EMULATION_DOWN = 0x08000000

button_mappings = {
   VPADButtons.VPAD_BUTTON_LEFT: libevdev.EV_KEY.BTN_DPAD_LEFT,
   VPADButtons.VPAD_BUTTON_RIGHT: libevdev.EV_KEY.BTN_DPAD_RIGHT,
   VPADButtons.VPAD_BUTTON_UP: libevdev.EV_KEY.BTN_DPAD_UP,
   VPADButtons.VPAD_BUTTON_DOWN: libevdev.EV_KEY.BTN_DPAD_DOWN,
   VPADButtons.VPAD_BUTTON_ZL: libevdev.EV_KEY.BTN_TL2,
   VPADButtons.VPAD_BUTTON_ZR: libevdev.EV_KEY.BTN_TR2,
   VPADButtons.VPAD_BUTTON_L: libevdev.EV_KEY.BTN_TL,
   VPADButtons.VPAD_BUTTON_R: libevdev.EV_KEY.BTN_TR,
   VPADButtons.VPAD_BUTTON_PLUS: libevdev.EV_KEY.BTN_START,
   VPADButtons.VPAD_BUTTON_MINUS: libevdev.EV_KEY.BTN_SELECT,
   VPADButtons.VPAD_BUTTON_HOME: libevdev.EV_KEY.BTN_MODE,
   VPADButtons.VPAD_BUTTON_STICK_R: libevdev.EV_KEY.BTN_THUMBR,
   VPADButtons.VPAD_BUTTON_STICK_L: libevdev.EV_KEY.BTN_THUMBL,
}

button_mappings_nintendo = { # maps button the way nintendo does
   VPADButtons.VPAD_BUTTON_A: libevdev.EV_KEY.BTN_EAST,
   VPADButtons.VPAD_BUTTON_B: libevdev.EV_KEY.BTN_SOUTH,
   VPADButtons.VPAD_BUTTON_X: libevdev.EV_KEY.BTN_NORTH,
   VPADButtons.VPAD_BUTTON_Y: libevdev.EV_KEY.BTN_WEST,
}

button_mappings_xbox = { # maps buttons the way microsoft does with the xbox
   VPADButtons.VPAD_BUTTON_A: libevdev.EV_KEY.BTN_SOUTH,
   VPADButtons.VPAD_BUTTON_B: libevdev.EV_KEY.BTN_EAST,
   VPADButtons.VPAD_BUTTON_X: libevdev.EV_KEY.BTN_WEST,
   VPADButtons.VPAD_BUTTON_Y: libevdev.EV_KEY.BTN_NORTH,
}

button_mappings = {**button_mappings, **button_mappings_nintendo} # switch nintendo to xbox to get xbox mappings

STICK_MAX = 32767 # i don't really know what value i should put here, any value seems to work...

print("Button mappings :", button_mappings)

device = libevdev.Device()
device.name = 'WiiU Gamepad (emulated)'

for button in button_mappings.values():
   device.enable(button)

# enabling Sticks
abs = libevdev.InputAbsInfo(minimum=-STICK_MAX, maximum=STICK_MAX)
device.enable(libevdev.EV_ABS.ABS_X, abs)
device.enable(libevdev.EV_ABS.ABS_Y, abs)
device.enable(libevdev.EV_ABS.ABS_RX, abs)
device.enable(libevdev.EV_ABS.ABS_RY, abs)


uinput = device.create_uinput_device()
print('device is now at {}'.format(uinput.devnode))

time.sleep(0.5) # apparently needed on some systems

# UDP socket setup
s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
s.bind(('0.0.0.0', 4242))

while True:
   msg = s.recvfrom(2048) # get data from wiiu

   # convert data to dict
   dict_str = msg[0].decode("UTF-8")
   data = ast.literal_eval(dict_str)
   wiiu_gamepad_data = data["wiiUGamePad"]
   #print(wiiu_gamepad_data)

   events = []

   for vpad_button in VPADButtons:   
         try:
            if wiiu_gamepad_data["hold"] & int(vpad_button):
               #print("pressed ", vpad_button)
               events.append(libevdev.InputEvent(button_mappings[vpad_button], value=1))
            else:
               events.append(libevdev.InputEvent(button_mappings[vpad_button], value=0)) 
         except KeyError:
            pass

   stick_values = {
      "l_stick_x": int(wiiu_gamepad_data["lStickX"] * STICK_MAX),
      "l_stick_y": int(wiiu_gamepad_data["lStickY"] * STICK_MAX),
      "r_stick_x": int(wiiu_gamepad_data["rStickX"] * STICK_MAX),
      "r_stick_y": int(wiiu_gamepad_data["rStickY"] * STICK_MAX),
   }

   # add stick values
   events.append(libevdev.InputEvent(libevdev.EV_ABS.ABS_X, value=stick_values["l_stick_x"]))
   events.append(libevdev.InputEvent(libevdev.EV_ABS.ABS_Y, value=stick_values["l_stick_y"]))
   events.append(libevdev.InputEvent(libevdev.EV_ABS.ABS_RX, value=stick_values["r_stick_x"]))
   events.append(libevdev.InputEvent(libevdev.EV_ABS.ABS_RY, value=stick_values["r_stick_y"]))
   
   events.append(libevdev.InputEvent(libevdev.EV_SYN.SYN_REPORT, 0)) # i think it's needed to register the inputs

   uinput.send_events(events)
